﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Airbus
{
    public partial class FormMapWithSetPlanes : Form
    {
        //словарь для выпадающего списка
        private readonly Dictionary<string, AbstractMap> _mapsDict = new()
        {
            {"Простая карта", new SimpleMap() },
            {"Буря в пустыне", new DesertStormMap() },
            {"Звёздные войны", new StarWarsMap() }
        };

        //объект от коллекции карт
        private readonly MapsCollection _mapsCollection;

        public FormMapWithSetPlanes()
        {
            InitializeComponent();
            _mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
            comboBoxSelectorMap.Items.Clear();
            foreach (var element in _mapsDict)
            {
                comboBoxSelectorMap.Items.Add(element.Key);
            }
        }

        //заполнение ListBoxMaps
        private void ReloadMaps()
        {
            int index = listBoxMaps.SelectedIndex;

            listBoxMaps.Items.Clear();

            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
            }

            if (listBoxMaps.Items.Count > 0 && (index == -1 || index >= listBoxMaps.Items.Count))
            {
                listBoxMaps.SelectedIndex = 0;
            }
            else if (listBoxMaps.Items.Count > 0 && index > -1 && index < listBoxMaps.Items.Count)
            {
                listBoxMaps.SelectedIndex = index;
            }
        }

        //добавление карты
        private void ButtonAddMap_Click_1(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);

                return;
            }

            if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("Нет такой карты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);

                return;
            }

            _mapsCollection.AddMap(textBoxNewMapName.Text,
           _mapsDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
        }

        //Выбор карты
        private void ListBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }

        // Удаление карты
        private void buttonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить карту {listBoxMaps.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ?? string.Empty);

                ReloadMaps();
            }
        }

        //добавление объекта
        private void ButtonAddPlane_Click(object sender, EventArgs e)
        {
            if(_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty] == null)
            {
                return;
            }

            Form1 form = new();

            if(form.ShowDialog() == DialogResult.OK)
            {
                DrawningObjectPlane plane = new(form.SelectedPlane);

                if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty] + plane != -1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty].ShowSet();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }

        //удаление объекта
        private void ButtonRemovePlane_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }

            if(MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, 
                MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }

            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);

            if(_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty] - pos != null)
            {
                MessageBox.Show("Объект удалён");
                pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty].ShowSet();
            } 
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        //вывод набора
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if(_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty] == null)
            {
                return;
            }

            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty].ShowSet();
        }

        //вывод карты
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty] == null)
            {
                return;
            }

            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty].ShowOnMap();
        }

        //перемещение
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if(_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty] == null)
            {
                return;
            }

            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;

            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }

            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? String.Empty].MoveObject(dir);
        }
    }
}
